#pragma once


#include "Point3d.h"
#include "ANN/ANN.h"
#include <vector>

using namespace std;


class UnstructuredGrid3D;





class PointSearch										//Simple class to search closest points in a point-cloud
{														//Uses internally the fast KD-tree method from the ANN library
public:
	
				PointSearch(UnstructuredGrid3D& grid,int Nmax);	
														//Initialize the search engine to work with the vertices of the given grid. 
														//Nmax gives the maximum number of closest-points we are allowed to search for.

			   ~PointSearch();	

void			closest(int N,const Point3d& p,vector<int>& indexes,vector<float>& distances);
														//Returns the N-closest grid-vertices to a search point 'p'. N must be smaller than Nmax (see constructor).
														//Vertex indices (in the grid) and their distances to 'p' are returned in the indexes[]
														//and distances[] arrays, in increasing order of distance to the search point 'p'.


protected:

UnstructuredGrid3D&	grid;								//The grid over whose vertices we are searching

ANNidxArray			nn_idx;								//Buffer for getting the nearest-neighbors from the ANN library
ANNdistArray		nn_distances;						//Buffer for getting distances to nearest-neighbors from the ANN library
ANNpointArray		data_pts;							//Buffer for storing 3D grid-vertex coordinates for the ANN library
ANNkd_tree*			kdt;								//KD-tree constructed by the ANN library for fast searching
};


